package ttt.isatm.controller;

import java.awt.Point;
import java.util.Random;

import ttt.isatm.model.TTTGameLogic;
import ttt.isatm.model.TTTPlayer;
import ttt.isatm.view.MainUIFrame;

/* Controller part of the MVC architecture 
 * This class talks to the View and Model side in order to pass information between the two
 */
public class TTTController {
	TTTGameLogic gameLogic;
	MainUIFrame mainFrame;
	
	public TTTController() {
		super();
	}
	
	// init model side with information received from the View
	public void initModel(String player1, String player2) {
		TTTPlayer pl1 = new TTTPlayer(player1, "X");
		TTTPlayer pl2 = new TTTPlayer(player2, "O");
		
		if((new Random()).nextBoolean())
			pl1.setCurrent(true);
		else 
			pl2.setCurrent(true);
		
		// init the game logic which represents the model side
		gameLogic = new TTTGameLogic(pl1, pl2);
	}
	
	// this method is to be used by the UI side to notify a move on the board
	public void notify(Point p) {
		doMove(p);
		if(gameLogic.getWinner() != null) {
			notify(gameLogic.getWinner());
		}
		gameLogic.setCurrentPlayer();
	}
	
	// get the curent marker used by player in the board
	public String getCurrentMarker() {
		return gameLogic.getCurrentPlayer().getMarker();
	}
	
	// get next marker that is representative for the next player on the board
	public String getNextMarker() {
		return gameLogic.getNextPlayer().getMarker();
	}
	
	// this method is to be used by the Controller side to notify the end of game and 
	// the winning player
	private void notify(TTTPlayer p) {
		mainFrame.displayGameScores(p.getName());
	}
	
	// apply move on the model side
	private void doMove(Point p) {
		gameLogic.move(p.x, p.y);
	}

	//  get & set
	public TTTGameLogic getGameLogic() {
		return gameLogic;
	}

	public void setGameLogic(TTTGameLogic gameLogic) {
		this.gameLogic = gameLogic;
	}

	public MainUIFrame getMainFrame() {
		return mainFrame;
	}

	public void setMainFrame(MainUIFrame mainFrame) {
		this.mainFrame = mainFrame;
	}
}
