package ttt.isatm.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GraphicsConfiguration;
import java.awt.HeadlessException;
import java.awt.Point;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;

import ttt.isatm.controller.TTTController;
import ttt.isatm.view.gui.GameBoardJPanel;
import ttt.isatm.view.gui.GameStartUpJPanel;
import ttt.isatm.view.gui.InfoJPanel;
import ttt.isatm.view.gui.ScoresJPanel;
import ttt.isatm.view.gui.custom.TTTJButton;

/*
 * View side of the MVC architecture; this is the main window
 */
public class MainUIFrame extends JFrame implements MainUIFrameListener {

	// player info
	String player1;
	String player2;

	// connection to controller; a simplification of the observer pattern
	TTTController controller;

	public MainUIFrame() throws HeadlessException {
		super();
		// TODO Auto-generated constructor stub
	}

	public MainUIFrame(GraphicsConfiguration gc) {
		super(gc);
		// TODO Auto-generated constructor stub
	}

	public MainUIFrame(String title, GraphicsConfiguration gc) {
		super(title, gc);
		// TODO Auto-generated constructor stub
	}

	public MainUIFrame(String title) throws HeadlessException {
		super(title);
		// TODO Auto-generated constructor stub
	}

	// first shown frame 
	public void initFrame() {
		//2. Optional: What happens when the frame closes?
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		//3. Create components and put them in the frame.
		//...create emptyLabel...
		getContentPane().add(new GameStartUpJPanel(this), BorderLayout.CENTER);

		//4. Size the frame.
		setSize(300,300);
		//frame.pack();


		//5. Show it.
		setVisible(true);
	}

	// after player info is submited show the game board area
	public void displayGameArea() {
		// get the container of the framework and delete all that is on it
		getContentPane().removeAll();
		
		// create the gameboard
		GameBoardJPanel gbjp = new GameBoardJPanel();
		
		// create the information panel (player info: marker, color, name)
		InfoJPanel ijp = new InfoJPanel(player1+" : " + controller.getCurrentMarker(),
				player2 + " : " + controller.getNextMarker());

		// link the gameboard with the mainframe in order to be able to notify the controler of new move and to get info about
		// current player
		gbjp.addMainUIFrameListener(this);
		
		// link gameboard with information panel so that status gets updated with the actual evolution of the game
		gbjp.addChangeListener(ijp);

		// add panels to the main frame's panel
		getContentPane().add(gbjp, BorderLayout.CENTER);
		getContentPane().add(ijp, BorderLayout.EAST);

		// show it.
		setVisible(true);
	}

	// when a winner is discovered display the info
	public void displayGameScores(String winner) {
		getContentPane().removeAll();

		String looser = "";
		if(player1.equals(winner))
			looser = player2;
		else
			looser = player1;
		ScoresJPanel scjp = new ScoresJPanel(winner, looser);

		getContentPane().add(scjp, BorderLayout.CENTER);

		setVisible(true);
	}

	public static void main(String[] args) {
		//0. create logic and controller
		TTTController controller = new TTTController();

		//1. Create the frame.
		final MainUIFrame frame = new MainUIFrame("FrameDemo");
		frame.initFrame();
		frame.setController(controller);
		controller.setMainFrame(frame);
	}

	// listener interface methods used by other components (i.e. controller) to react
	//
	// to some interesting View events and reactions
	@Override
	public void initPlayers(String player1, String player2) {
		// TODO Auto-generated method stub
		this.player1 = player1;
		this.player2 = player2;

		controller.initModel(player1, player2);
		displayGameArea();		
	}

	@Override
	public String getCurrentMarker() {
		// TODO Auto-generated method stub
		return controller.getCurrentMarker();		
	}

	@Override
	public void notifyMove(Point p) {
		controller.notify(p);
	}

	@Override
	public String getWinner() {
		// TODO Auto-generated method stub
		return null;
	}

	public void setController(TTTController controller) {
		this.controller = controller;
	}
}
